package edu.iastate.sdmay2025.distributors;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import edu.iastate.sdmay2025.predictions.Predictions;
import edu.iastate.sdmay2025.predictions.PredictionsRepo;

@RestController
@CrossOrigin(origins= {"*"})
public class DistributorsController {
	
	@Autowired
	DistributorsRepo DistributorsRepo;
	
	@Autowired
	PredictionsRepo predictionsRepo;
	
	@ResponseBody
	@RequestMapping(method = RequestMethod.GET, path="/distributors/{distributionCenterID}")
	public Collection<DistributorsNoJoin> getDistributorsWithCenterID(@PathVariable int distributionCenterID) {
		Collection<DistributorsNoJoin> result = DistributorsRepo.getDistributorsWithCenterID(DistributorsNoJoin.class, distributionCenterID);
		return result;
	}
	
	@ResponseBody
	@RequestMapping(method = RequestMethod.GET, path="/distributors/withProducts/{distributionCenterID}")
	public Collection<DistributorsNoJoin> getDistributorsWithProductsCenterID(@PathVariable int distributionCenterID) {
		Collection<DistributorsNoJoin> result = DistributorsRepo.getDistributorsWithProductsCenterID(DistributorsNoJoin.class, distributionCenterID);
		return result;
	}
	
	@ResponseBody
	@RequestMapping(method = RequestMethod.GET, path="/distributors/withProductsToOrder/{distributionCenterID}")
	public Collection<DistributorsNoJoin> getDistributorsWithProductsToOrderCenterID(@PathVariable int distributionCenterID) {
		Collection<DistributorsNoJoin> result = DistributorsRepo.getDistributorsWithProductsToOrderCenterID(DistributorsNoJoin.class, distributionCenterID);
		return result;
	}	
	
	@ResponseBody
	@RequestMapping(method = RequestMethod.GET, path="/distributor/{distributorID}")
	public Collection<DistributorsNoJoin> getDistributorsWithID(@PathVariable int distributorID) {
		Collection<DistributorsNoJoin> result = DistributorsRepo.getDistributorsWithID(DistributorsNoJoin.class, distributorID);
		return result;
	}
	
	@ResponseBody
	@RequestMapping(method = RequestMethod.GET, path="/distributor/products/{distributorID}")
	public  List<DistributorProductsDto> getDistributorProducts(@PathVariable int distributorID) {
		List<DistributorProductsDto> result = DistributorsRepo.getDistributorProducts(distributorID);
		
		for( DistributorProductsDto product : result ) {
			Optional<Predictions> prediction = predictionsRepo.findTop1BySkusIdOrderByDateDesc( product.getSkuID() );
			
			if( prediction.isPresent() ) {
				product.setQtyToOrder(prediction.get().getQtyToOrder());
			}
		}
		return result;
	}
	
	@ResponseBody
	@RequestMapping(method = RequestMethod.GET, path="/distributor/products/withpredictions/{distributorID}")
	public  List<DistributorProductsDto> getDistributorProductsWithPredictions(@PathVariable int distributorID) {
		List<DistributorProductsDto> result = this.getDistributorProducts(distributorID);
		
		Iterator<DistributorProductsDto> iterator = result.iterator();
		DistributorProductsDto currentProd;
		while( iterator.hasNext() ) {
			currentProd = iterator.next();
			if( currentProd.getQtyToOrder() == 0 ) {
				iterator.remove();
			}
		}

		return result;
	}
	
	@ResponseBody	
	@RequestMapping(method = RequestMethod.GET, path="/distributor/schedule/{distributorID}")
	public List<DistributorScheduleDTO> getDistributorSchedule(@PathVariable int distributorID) {
		List<DistributorScheduleDTO> result = DistributorsRepo.getDistributorSchedule(distributorID);
		return result;
	}

	@ResponseBody	
	@RequestMapping(method = RequestMethod.GET, path="/distributors/scheduleAll/{distributionCenterID}")
	public List<DistributorScheduleDTO> getDistributorsScheduleAll(@PathVariable int distributionCenterID) {
		List<DistributorScheduleDTO> result = DistributorsRepo.getDistributorsScheduleAll(distributionCenterID);
		return result;
	}
}
