package edu.iastate.sdmay2025.distributors;

import java.util.Collection;
import java.util.List;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

public interface DistributorsRepo extends JpaRepository<Distributors, Integer> {

	@Query(value="SELECT * FROM distributors", nativeQuery=true)
	Collection<Distributors> getDistributors();
	
	@Query(value="SELECT * FROM distributors", nativeQuery=true)
	<T> Collection<T> getDistributors(Class<T> type);
	
	@Query(value="SELECT * FROM distributors WHERE distribution_center_id = :distributionCenterID and distributors.distributor_type = 0 order by distributors.id", nativeQuery=true)
	Collection<Distributors> getDistributorsWithCenterID(@Param("distributionCenterID") int distributionCenterID);
	
	@Query(value="SELECT * FROM distributors WHERE distribution_center_id = :distributionCenterID and distributors.distributor_type = 0 order by distributors.id", nativeQuery=true)
	<T> Collection<T> getDistributorsWithCenterID(Class<T> type,@Param("distributionCenterID") int distributionCenterID);
	
	@Query(value="SELECT * FROM distributors WHERE distribution_center_id = :distributionCenterID and distributors.distributor_type = 0 and distributors.deleted = false and distributors.id in (Select distributors.id from distributors \r\n" + 
			"inner join purchasing_sizes\r\n" + 
			"	on distributors.id = purchasing_sizes.distributor_id\r\n" + 
			"inner join skus\r\n" + 
			"	on purchasing_sizes.sku_id = skus.id)"
			+ " order by distributors.id", nativeQuery=true)
	Collection<Distributors> getDistributorsWithProductsCenterID(@Param("distributionCenterID") int distributionCenterID);
	
	@Query(value="SELECT * FROM distributors WHERE distribution_center_id = :distributionCenterID and distributors.distributor_type = 0 and distributors.deleted = false and distributors.id in (Select distributors.id from distributors \r\n" + 
			"inner join purchasing_sizes\r\n" + 
			"	on distributors.id = purchasing_sizes.distributor_id\r\n" + 
			"inner join skus\r\n" + 
			"	on purchasing_sizes.sku_id = skus.id)"
			+ " order by distributors.id", nativeQuery=true)
	<T> Collection<T> getDistributorsWithProductsCenterID(Class<T> type,@Param("distributionCenterID") int distributionCenterID);
	
	@Query( value="SELECT * FROM distributors WHERE distribution_center_id = :distributionCenterID and distributors.distributor_type = 0 and distributors.deleted = false \r\n" + 
			"and distributors.id in (Select distributors.id from distributors \r\n" + 
			"inner join purchasing_sizes\r\n" + 
			"	on distributors.id = purchasing_sizes.distributor_id\r\n" + 
			"inner join skus\r\n" + 
			"	on purchasing_sizes.sku_id = skus.id\r\n" + 
			"inner join predictions\r\n" + 
			"	on skus.id = predictions.sku_id\r\n" + 
			"where predictions.qty_to_order > 0)\r\n" + 
			"order by distributors.id", nativeQuery=true)
	Collection<Distributors> getDistributorsWithProductsToOrderCenterID( @Param("distributionCenterID") int distributionCenterID);
	
	@Query( value="SELECT * FROM distributors WHERE distribution_center_id = :distributionCenterID and distributors.distributor_type = 0 and distributors.deleted = false \r\n" + 
			"and distributors.id in (Select distributors.id from distributors \r\n" + 
			"inner join purchasing_sizes\r\n" + 
			"	on distributors.id = purchasing_sizes.distributor_id\r\n" + 
			"inner join skus\r\n" + 
			"	on purchasing_sizes.sku_id = skus.id\r\n" + 
			"inner join predictions\r\n" + 
			"	on skus.id = predictions.sku_id\r\n" + 
			"where predictions.qty_to_order > 0)\r\n" + 
			"order by distributors.id", nativeQuery=true)
	<T> Collection<T> getDistributorsWithProductsToOrderCenterID( Class<T> type, @Param("distributionCenterID") int distributionCenterID);
	
	@Query(value="SELECT * FROM distributors WHERE id = :distributorID and distributors.distributor_type = 0", nativeQuery=true)
	Collection<Distributors> getDistributorsWithID(@Param("distributorID") int distributorID);
	
	@Query(value="SELECT * FROM distributors WHERE id = :distributorID and distributors.distributor_type = 0", nativeQuery=true)
	<T> Collection<T> getDistributorsWithID(Class<T> type,@Param("distributorID") int distributorID);
	
	@Query(value="Select new edu.iastate.sdmay2025.distributors.DistributorProductsDto(dist.id, dist.name, pro.name, bre.name, sku.id, sku.baseSkuId, pur.costOfGoods, pur.onHandInventoryTargetLevel, psize.numberOfPacks, psize.packName, psize.units, psize.sizeOfMeasurement, psize.unitOfMeasurement, psize.name) " +
			"From distributors dist " +
			"inner join dist.purchasingSizes pur " +
			"inner join pur.skus sku " +
			"inner join sku.products pro " +
			"inner join pro.breweries bre " +
			"inner join sku.productSizes psize " +
			"where dist.distributorType = 0 and dist.id = :distributorID " +
			"order by dist.name, sku.baseSkuId "
			, nativeQuery=false)
	List<DistributorProductsDto> getDistributorProducts(@Param("distributorID") int distributorID);

	@Query(value="SELECT new edu.iastate.sdmay2025.distributors.DistributorScheduleDTO(dist.id, dist.name, oday.weekBasis, oday.daysToDelivery, oday.daysToDeliveryEnd) " + 
			"FROM distributors dist " + 
			"INNER JOIN dist.distributorOrderDays oday " +  
			"WHERE " + 
			"dist.deleted is false AND " + 
			"dist.id = :distributorID " +
			"GROUP BY dist.id, dist.name, oday.weekBasis, oday.daysToDelivery, oday.daysToDeliveryEnd " +
			"ORDER BY dist.id", 
			nativeQuery=false )
	List<DistributorScheduleDTO> getDistributorSchedule(@Param("distributorID") int distributorID);
	
	@Query(value="SELECT new edu.iastate.sdmay2025.distributors.DistributorScheduleDTO(dist.id, dist.name, oday.weekBasis, oday.daysToDelivery, oday.daysToDeliveryEnd) " + 
			"FROM distributors dist " + 
			"INNER JOIN dist.distributorOrderDays oday " + 
			"INNER JOIN dist.purchasingSizes pur " +
			"INNER JOIN pur.skus " +
			"WHERE " + 
			"dist.deleted is false AND " + 
			"dist.distributionCenters.id = :distributionCenterID " +
			"GROUP BY dist.id, dist.name, oday.weekBasis, oday.daysToDelivery, oday.daysToDeliveryEnd " +
			"ORDER BY dist.id", 
			nativeQuery=false )
	List<DistributorScheduleDTO> getDistributorsScheduleAll(@Param("distributionCenterID") int distributionCenterID);
	
}
