package edu.iastate.sdmay2025.finalizedpurchases;

import java.util.List;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

/**
 * FInalized Purchases Repository
 * 
 * @author Andrew Smith
 */
public interface FinalizedPurchasesRepo extends JpaRepository<FinalizedPurchases, Integer> {

	/**
	 * A query to get the Missed sales items for a sku from the database.
	 * 
	 * This is not a valid PSQL query. It is based on one from the wiki
	 * (https://git.ece.iastate.edu/sd/sdmay20-25/-/wikis/tables#optimized-query-for-spring-on-inventory-history-of-a-single-sku-in-a-distribution-center)
	 * 
	 * @param sku sku to get the missed sales items historical data
	 * @return Historical Data on Missed Sales Based given a SKU ID with some identifying data
	 */
	@Query(value="SELECT new edu.iastate.sdmay2025.finalizedpurchases.FinalizedPurchasesDTO(sku.id, fin.createdAt, miss.count) " + 
			"FROM finalized_purchases fin " + 
			"INNER JOIN fin.missedItems miss " +
			"INNER JOIN miss.purchasingSizes pur " +
			"INNER JOIN pur.skus sku " +
			"WHERE " + 
			"fin.deleted = false AND " +
			"fin.orderStatus = 0 AND " +
			"sku.id = :sku " +
			"ORDER BY fin.createdAt ", 
			nativeQuery=false )
	List<FinalizedPurchasesDTO> getMissedItemsBySku(@Param("sku") long sku);
	
}