package edu.iastate.sdmay2025.inventorysnapshots;

import java.util.Collection;
import java.util.List;

import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;

/**
 * Inventory Snapshots Repository
 * 
 * @author Sam Stifter, Andrew Smith
 */
public interface InventorySnapshotsRepo extends JpaRepository<InventorySnapshots, Integer> {

	/**
	 * A query to get the stock history for a sku from the database.
	 * 
	 * This is not a valid PSQL query. It is based on one from the wiki
	 * (https://git.ece.iastate.edu/sd/sdmay20-25/-/wikis/tables#optimized-query-for-spring-on-inventory-history-of-a-single-sku-in-a-distribution-center)
	 * 
	 * @param sku sku to get the inventory levels for
	 * @return Daily inventory history
	 */
	@Query(value="SELECT new edu.iastate.sdmay2025.inventorysnapshots.InventorySnapshotsDto( fin.createdAt, inv.count) " + 
			"FROM inventory_snapshots inv " + 
			"INNER JOIN inv.financialRecords fin " + 
			"INNER JOIN inv.distributionCentersSkus dist " + 
			"WHERE " + 
			"fin.distributionCenterId = 1 AND " + 
			"dist.skus.id = :sku ", 
			nativeQuery=false )
	List<InventorySnapshotsDto> getSkuInventorySnapshots(@Param("sku") long sku);
	
}
