package edu.iastate.sdmay2025.predictions;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import edu.iastate.sdmay2025.skus.Skus;
import edu.iastate.sdmay2025.skus.SkusRepository;

@RestController
@CrossOrigin(origins= {"*"})
public class PredictionsController {
	
	@Autowired
	PredictionsRepo predictionsRepo;
	
	@Autowired
	SkusRepository skusRepository;
	
	@RequestMapping(value = "/predictions/add", method = RequestMethod.POST)
	public ResponseEntity<PredictionDTO> add(@RequestBody PredictionDTO prediction) {
		Predictions predictionsDB = new Predictions();
		
		Optional<Skus> base_sku = skusRepository.findById(prediction.getSku_id());
		
		if(prediction != null) {
			predictionsDB.setDate(new Timestamp(System.currentTimeMillis()));
			predictionsDB.setQtyToOrder(prediction.getQty_to_order());
			if( base_sku.isPresent() ) predictionsDB.setSkus(base_sku.get());
			
			predictionsDB = predictionsRepo.save(predictionsDB);
			prediction = new PredictionDTO( predictionsDB );
		}
		
		return new ResponseEntity<PredictionDTO>(prediction, HttpStatus.OK);
	}
	
	@RequestMapping(value = "/predictions/sku/{sku}", method = RequestMethod.GET)
	public ResponseEntity<List<PredictionDTO>> findBySku(@PathVariable long sku) {
		List<Predictions> result = predictionsRepo.findBySkusId(sku);
		List<PredictionDTO> dtoResult = new ArrayList<>();
		
		for( Predictions prediction : result ) {
			dtoResult.add(new PredictionDTO(prediction));
		}
		
		return new ResponseEntity<List<PredictionDTO>>(dtoResult, HttpStatus.OK);
	}
	
	@RequestMapping(value = "/predictions/skus", method = RequestMethod.GET)
	public ResponseEntity<List<PredictionDTO>> findAll(){
		boolean replace = false;
		int index = -1;
		boolean copy = false;
		List<Predictions> result = predictionsRepo.findAll();
		List<PredictionDTO> dtoResult = new ArrayList<>();
		
		for( Predictions prediction : result ) {
			for(PredictionDTO dtoPre: dtoResult) {
				if(prediction.getSkus().getId() == dtoPre.getSku_id()) {
					copy = true;
					if(prediction.getDate().after(dtoPre.getDate())) {
						replace = true;
						index = dtoResult.indexOf(dtoPre);
						//dtoResult.add(dtoResult.indexOf(dtoPre), new PredictionDTO(prediction));
					}
				}
			}
			if(!copy) {
				dtoResult.add(new PredictionDTO(prediction));
			}else if(copy && replace) {
				dtoResult.remove(index);
				dtoResult.add(index, new PredictionDTO(prediction));
				index = -1;
			}
			copy = false;
			replace = false;
		}
		
		return new ResponseEntity<List<PredictionDTO>>(dtoResult, HttpStatus.OK);
	}
	
	@RequestMapping(value = "/predictions/latest/sku/{sku}", method = RequestMethod.GET)
	public ResponseEntity<PredictionDTO> findLatestBySku(@PathVariable long sku) {
		Optional<Predictions> result = predictionsRepo.findTop1BySkusIdOrderByDateDesc(sku);
		PredictionDTO dtoResult = new PredictionDTO( result.get() );
		
		return new ResponseEntity<PredictionDTO>(dtoResult, HttpStatus.OK);
	}
}
