export class DistributorEntity {
  id: number;
  name: string;
  orderMin: number;
  term: number;
  buffer: number;
  orderSchedule: {
    monday: boolean;
    tuesday: boolean;
    wednesday: boolean;
    thursday: boolean;
    friday: boolean;
    saturday: boolean;
    sunday: boolean;
  };
  extraData?: ExtraDistributorData;
  products: DistributorProduct[] | null;

  constructor(data: any) {
    this.id = data.id;
    this.name = data.name;
    this.orderMin = data.order_minimum || data.orderMin;
    this.term = data.term;
    this.buffer = data.procurement_buffer || data.buffer;
    if (data.orderSchedule) {
      this.orderSchedule = data.orderSchedule;
    } else {
      this.orderSchedule = {
        monday: data.monday,
        tuesday: data.tuesday,
        wednesday: data.wednesday,
        thursday: data.thursday,
        friday: data.friday,
        saturday: data.saturday,
        sunday: data.sunday,
      };
    }
    this.products = null;

    this.extraData =
      data.extraData ||
      new ExtraDistributorData({
        weekBasis: data.weekBasis,
        daysToDelivery: data.daysToDelivery,
        daysToDeliveryEnd: data.daysToDeliveryEnd,
        //startDate: data.startDate,
      });
  }
}

export class ExtraDistributorData {
  weekBasis: number;
  daysToDelivery: number;
  daysToDeliveryEnd: number;
  //startDate: moment.Moment | undefined;

  constructor(data: any) {
    this.weekBasis = data.weekBasis;
    this.daysToDelivery = data.daysToDelivery;
    this.daysToDeliveryEnd = data.daysToDeliveryEnd;
    //this.startDate = data.startDate ? moment(data.startDate) : undefined;
  }
}

export class DistributorProduct {
  key: number;
  cost: number;
  onHandInventoryTargetLevel: number;
  qtyToOrder: number;
  distName: string;
  productName: string;
  brandName: string;
  skuID: number;
  distBaseSkuID: number;
  distID: number;
  sizeName: string;
  purchaseCost: number;

  constructor(data: any) {
    this.key = data.key !== undefined ? data.key : -1;
    this.onHandInventoryTargetLevel =
      data.onHandInventoryTargetLevel !== undefined
        ? data.onHandInventoryTargetLevel
        : -1;
    this.qtyToOrder = data.qtyToOrder !== undefined ? data.qtyToOrder : -1;
    this.cost = data.costOfGoods !== undefined ? data.costOfGoods : -1;
    this.distName = data.distName !== undefined ? data.distName : "";
    this.productName = data.productName !== undefined ? data.productName : "";
    this.brandName = data.brandName !== undefined ? data.brandName : "";
    this.skuID = data.skuID !== undefined ? data.skuID : -1;
    this.distBaseSkuID =
      data.distBaseSkuID !== undefined ? data.distBaseSkuID : -1;
    this.distID = data.distID !== undefined ? data.distId : -1;
    this.sizeName = data.sizeName !== undefined ? data.sizeName : "";

    this.purchaseCost =
      this.qtyToOrder !== -1 && this.cost !== -1
        ? this.qtyToOrder * this.cost
        : 0;
  }
}

export class DistributorsGetRequest {
  regionId: number;
  onlyDistributorsWithProducts: boolean;

  constructor(data: any) {
    this.regionId = data.districtId;
    this.onlyDistributorsWithProducts = data.onlyDistributorsWithProducts;
  }
}

export class DistributorGetRequest {
  id: number;

  constructor(data: any) {
    this.id = data.id;
  }
}

export class DistributorProductsGetRequest {
  distributorId: number;

  constructor(data: any) {
    this.distributorId = data.distributorId;
  }
}
