import * as React from "react";
import { Table, Spin, message, Checkbox } from "antd";
import {
  IDistributorHandler,
  DistributorHandler,
} from "./Distributor.api.handler";
import {
  DistributorEntity,
  DistributorGetRequest,
  DistributorProduct,
  DistributorProductsGetRequest,
} from "./Distributor.api.types";
import { ColumnProps } from "antd/lib/table";
import { WEEK_DAYS } from "../../constants";

/**Function to build a string for order schedule days based on a distributor order schedule*/

export function createOrderScheduleString(orderSchedule: any): string {
  if (!orderSchedule) {
    return "-";
  }

  let orderDays: string[] = [];
  let orderDaysString: string = "";
  const orderScheduleList: boolean[] = [
    orderSchedule.monday,
    orderSchedule.tuesday,
    orderSchedule.wednesday,
    orderSchedule.thursday,
    orderSchedule.friday,
    orderSchedule.saturday,
    orderSchedule.sunday,
  ];

  //Construct array where only true day values remain
  for (let i = 0; i < orderScheduleList.length; i++) {
    if (orderScheduleList[i]) {
      orderDays.push(WEEK_DAYS[i]);
    }
  }

  for (let i = 0; i < orderDays.length; i++) {
    orderDaysString += orderDays[i];

    if (i !== orderDays.length - 1) {
      orderDaysString += ", ";
    }
  }
  if (orderDaysString === "") orderDaysString = "-";
  return orderDaysString;
}

const columns = (
  data: DistributorEntity,
  products: DistributorProduct[]
): ColumnProps<any>[] => [
  {
    title: data.id,
    children: [
      {
        title: (
          <div>
            <p>Distributer</p> <p>ProductID</p> <p>---</p>
            <p>ExernalID</p>
          </div>
        ),
        dataIndex: "productId",
        key: "productId",
      },
    ],
    key: data.id,
  },
  {
    title: (
      <>
        <h2>{data.name}</h2>
        <div
          style={{
            display: "flex",
            flexDirection: "row",
          }}
        >
          <div>
            <p>Order Method: </p>
            <p>Address: </p>
          </div>
          <div>
            <p>Active: {createOrderScheduleString(data.orderSchedule)}</p>
            <p>Order Due: </p>
          </div>
        </div>
      </>
    ),
    children: [
      {
        title: (
          <div>
            <p>Cost</p> <p>---</p> <p>Price</p>
          </div>
        ),
        dataIndex: "cost",
        key: "cost",
      },
      {
        title: "Product Info",
        dataIndex: "productInfo",
        key: "productInfo",
      },
      {
        title: "Reorder Threshold Units",
        dataIndex: "reorderThresholdUnits",
        key: "reorderThresholdUnits",
      },
      {
        title: "Order Up To Units",
        dataIndex: "orderUpToUnits",
        key: "orderUpToUnits",
      },
      {
        title: "OH Inventory (Base Units)",
        dataIndex: "baseUnits",
        key: "baseUnits",
      },
      {
        title: "On Hand Units En Route",
        dataIndex: "onHandUnitsInRoute",
        key: "onHandUnitsInRoute",
      },
    ],
    key: data.id,
  },
  {
    title: "Order Minimum",
    children: [
      {
        title: (
          <p>
            ${data.orderMin}/$
            <input type="number" readOnly value={1} style={{ width: "50px" }} />
          </p>
        ),
        children: [
          {
            title: "Amount Needed",
            dataIndex: "amountNeeded",
            key: "amountNeeded",
          },
          {
            title: "Amount To Order",
            dataIndex: "amountToOrder",
            key: "amountToOrder",
          },
        ],
      },
    ],
    key: data.id,
  },
  {
    title: "Term",
    children: [
      {
        title: (
          <input
            type="number"
            value={data.term || undefined}
            style={{ width: "50px" }}
          />
        ),
        children: [
          {
            title: "Order Spend",
            dataIndex: "orderSpend",
            key: "orderSpend",
          },
        ],
      },
    ],
    key: data.id,
  },
  {
    title: "Buffer",
    children: [
      {
        title: (
          <>
            <input
              type="number"
              value={data.buffer || undefined}
              style={{ width: "50px" }}
              readOnly
            />
            x
          </>
        ),
        children: [
          {
            title: "Missed Item Count",
            dataIndex: "missedItemCount",
            key: "missedItemCount",
          },
          {
            title: "Order Stats",
            dataIndex: "orderStats",
            key: "orderStats",
          },
        ],
        key: data.id,
      },
    ],
    key: data.id,
  },
];

export interface IDistributorTableProps {
  handler?: IDistributorHandler;
  distributorId: number;
  regionId: number;
}

interface IDistributorTableState {
  distributorEntity?: DistributorEntity;
  distributorProducts?: DistributorProduct[];
}

//deprecated
export default class DistributorTable extends React.Component<
  IDistributorTableProps,
  IDistributorTableState
> {
  static defaultProps = {
    handler: new DistributorHandler(),
  };

  state: IDistributorTableState = {
    distributorEntity: undefined,
    distributorProducts: [],
  };

  getDistributorAndProducts = () => {
    try {
      this.getDistributor(this.props.distributorId);
      this.getDistributorProducts(this.props.distributorId);
    } catch (e) {
      this.setState(() => {
        throw e;
      });
    }
  };

  componentDidMount = async () => {
    this.getDistributorAndProducts();
  };

  componentDidUpdate(prevProps: IDistributorTableProps) {
    if (this.props.distributorId !== prevProps.distributorId) {
      this.getDistributorAndProducts();
    }
  }

  getDistributor = async (id: number) => {
    let distributor: DistributorEntity;
    try {
      distributor = await this.props.handler!.get(
        new DistributorGetRequest({ id })
      );

      this.setState({
        distributorEntity: distributor,
      });
    } catch (err) {
      message.error(err.message);
    }
  };

  getDistributorProducts = async (id: number) => {
    let distributorProducts: DistributorProduct[];
    try {
      distributorProducts = await this.props.handler!.getProducts(
        new DistributorProductsGetRequest({ distributorId: id })
      );
      const productRows = distributorProducts.map(
        (product: DistributorProduct, i: number) => {
          return {
            productId: (
              <div>
                <span>
                  <Checkbox />{" "}
                  <span style={{ color: "blue" }}>{product.skuID}</span>
                </span>
                <hr />
                <p>{product.distBaseSkuID}</p>
              </div>
            ),
            cost: (
              <div>
                <p>
                  ${" "}
                  <input
                    type="number"
                    min="0.01"
                    step="0.01"
                    readOnly
                    value={product.cost}
                    style={{ width: "50px" }}
                  />
                </p>
                <p>
                  ${" "}
                  <input
                    type="number"
                    min="0.01"
                    step="0.01"
                    readOnly
                    value={product.cost}
                    style={{ width: "50px" }}
                  />
                </p>
              </div>
            ),
            // productInfo: "Product Info Here",
            // reorderThresholdUnits: (
            //   <div>
            //     <input
            //       readOnly
            //       value={-1}
            //       type="number"
            //       step={0.01}
            //       style={{ width: "50px" }}
            //     />
            //   </div>
            // ),
            // orderUpToUnits: (
            //   <div>
            //     <input
            //       readOnly
            //       value={-1}
            //       type="number"
            //       step={0.01}
            //       style={{ width: "50px" }}
            //     />
            //   </div>
            // ),
            // baseUnits: (
            //   <>
            //     {product.baseUnits}({product.baseUnits})
            //   </>
            // ),
            // onHandUnitsInRoute: product.onHandUnitsInRoute,
            // amountNeeded: product.ammountNeeded,
            // amountToOrder: (
            //   <div>
            //     <input
            //       readOnly
            //       value={product.ammountToOrder}
            //       type="number"
            //       style={{ width: "50px" }}
            //     />
            //   </div>
            // ),
            // orderSpend: <span>${product.orderSpend}</span>,
            // missedItemCount: product.missedItemCount,
            // _id: product.key,
            // orderStats: (
            //   <Icon type="info-circle" theme="twoTone" twoToneColor="#eb2f96" />
            // )
          };
        }
      );
      this.setState({
        distributorProducts: distributorProducts,
      });
    } catch (err) {
      message.error(err.message);
    }
  };

  public render() {
    if (!this.state.distributorEntity) {
      return <Spin />;
    }

    if (!this.state.distributorProducts) {
      return <Spin />;
    }

    return (
      <div>
        {/*https://ant.design/components/table/#header*/}
        <Table
          columns={columns(
            this.state.distributorEntity,
            this.state.distributorProducts
          )}
          dataSource={this.state.distributorProducts}
          bordered
          size="middle"
          scroll={{ x: "calc(700px + 50%)", y: 240 }}
          rowKey={(record) => record.distID + record.distBaseSkuID}
        />
      </div>
    );
  }
}
