import axios from "axios";
import { Region } from "./ProcurementTool.api.types";
import { Error } from "../util/Error";
import { DistributorEntity } from "../DistributorTable/Distributor.api.types";
import { getAllRegions } from "../../apiEngine";
import { BASE_API_URL } from "../../constants";

export interface IProcurementToolHandler {
  getAllRegions(): Promise<Region[]>;
  getAllDistributorsByRegion(regionId: number): Promise<DistributorEntity[]>;
}

export class ProcurementToolHandler implements IProcurementToolHandler {
  async getAllRegions(): Promise<Region[]> {
    return await getAllRegions();
  }

  async getAllDistributorsByRegion(
    regionId: number
  ): Promise<DistributorEntity[]> {
    const url = `${BASE_API_URL}/distributors/${regionId}`;
    let response;
    try {
      response = await axios.get(url);
      let distributors: DistributorEntity[] = response.data.map(
        (d: any) => new DistributorEntity(d)
      );
      distributors.sort((a: DistributorEntity, b: DistributorEntity) =>
        a.name.localeCompare(b.name)
      );
      return distributors;
    } catch (e) {
      throw new Error({
        error: e,
        message:
          "There was a problem fetching regions. Try connecting to the ISU VPN or running backend locally",
      });
    }
  }
}
