import * as React from "react";
import DistributorTable from "../DistributorTable/DistributorTable";
import { Select } from "antd";
import { DistributorEntity } from "../DistributorTable/Distributor.api.types";
import { Region } from "./ProcurementTool.api.types";
import {
  ProcurementToolHandler,
  IProcurementToolHandler,
} from "./ProcurementTool.api.handler";

const { Option } = Select;

export interface IProcurementToolProps {
  handler?: IProcurementToolHandler;
}
interface IProcurementToolState {
  regions: Region[];
  selectedRegionId: undefined | number;
  selectedDistributorId: undefined | number;
  distributors: DistributorEntity[];
}

export default class ProcurementTool extends React.Component<
  IProcurementToolProps,
  IProcurementToolState
> {
  static defaultProps = {
    handler: new ProcurementToolHandler(),
  };

  state: IProcurementToolState = {
    regions: [],
    selectedRegionId: undefined,
    selectedDistributorId: undefined,
    distributors: [],
  };
  componentDidMount = async () => {
    const { selectedRegionId } = this.state;

    const regionEntities: Region[] = await this.props.handler!.getAllRegions();

    let regionId: number | undefined = undefined;
    if (regionEntities.length > 0) {
      regionId = selectedRegionId ? selectedRegionId : regionEntities[0].id;
      this.getDistributorList(regionId);
    }
    this.setState({
      regions: regionEntities,
      selectedRegionId: regionId,
    });
  };

  getDistributorList = async (regionId: number) => {
    const distributors: DistributorEntity[] = await this.props.handler!.getAllDistributorsByRegion(
      regionId
    );
    this.setState({
      distributors: distributors,
      selectedDistributorId:
        distributors.length > 0 ? distributors[0].id : undefined,
    });
  };

  handleRegionChange = (val: any) => {
    //Update distributors
    this.getDistributorList(val);
    this.setState({
      selectedRegionId: val,
    });
  };

  handleDistributorChange = (val: any) => {
    this.setState({
      selectedDistributorId: val,
    });
  };

  render() {
    const { regions, distributors } = this.state;
    return (
      <div>
        <Select
          style={{ width: 120 }}
          defaultValue=""
          value={
            regions.length > 0
              ? this.state.selectedRegionId || regions[0].name
              : ""
          }
          onChange={this.handleRegionChange}
          filterOption={(input, option: any) =>
            option.props.children.toLowerCase().indexOf(input.toLowerCase()) >=
            0
          }
          optionFilterProp="children"
          showSearch
        >
          {regions.map((region: Region, i: number) => (
            <Option value={region.id} key={i}>
              {region.name}
            </Option>
          ))}
        </Select>
        <Select
          style={{ width: 200 }}
          defaultValue=""
          value={
            distributors.length > 0
              ? this.state.selectedDistributorId || distributors[0].name
              : ""
          }
          onChange={this.handleDistributorChange}
          filterOption={(input, option: any) =>
            option.props.children.toLowerCase().indexOf(input.toLowerCase()) >=
            0
          }
          optionFilterProp="children"
          showSearch
        >
          {distributors.map((distributor: DistributorEntity, i: number) => (
            <Option value={distributor.id} key={i}>
              {distributor.name}
            </Option>
          ))}
        </Select>

        {this.state.selectedDistributorId && this.state.selectedRegionId && (
          <DistributorTable
            distributorId={this.state.selectedDistributorId}
            regionId={this.state.selectedRegionId}
          />
        )}
      </div>
    );
  }
}
