import axios from "axios";

import { Error } from "../util/Error";
import { OrderHistoryLevel, MissedOrder } from "./ProductGraph.api.types";
import { BASE_API_URL } from "../../constants";

export interface IProductGraphHandler {
  getProductOrderHistoryBySKU(skuID: number): Promise<OrderHistoryLevel[]>;
  getProductMissedItemHistoryBySKU(skuID: number): Promise<MissedOrder[]>;
}

export class ProductGraphHandler implements IProductGraphHandler {
  async getProductOrderHistoryBySKU(
    skuID: number
  ): Promise<OrderHistoryLevel[]> {
    const url = `${BASE_API_URL}/sku_hist/${skuID}`;
    let response;
    try {
      response = await axios.get(url);
      const orderHistoryLevels: OrderHistoryLevel[] = response.data.map(
        (levelData: any) => new OrderHistoryLevel(levelData)
      );
      return orderHistoryLevels;
    } catch (e) {
      throw new Error({
        error: e,
        message:
          "There was a problem fetching Order History Levels. Try connecting to the ISU VPN or running backend locally",
      });
    }
  }

  async getProductMissedItemHistoryBySKU(
    skuID: number
  ): Promise<MissedOrder[]> {
    const url = `${BASE_API_URL}/missedItemsBySku/${skuID}`;
    let response;
    try {
      response = await axios.get(url);
      const missedOrders: MissedOrder[] = response.data.map(
        (missedOrderData: any) => new MissedOrder(missedOrderData)
      );
      return missedOrders;
    } catch (e) {
      throw new Error({
        error: e,
        message:
          "There was a problem fetching Missed Orders. Try connecting to the ISU VPN or running backend locally",
      });
    }
  }
}
