import * as React from "react";
import Plot from "react-plotly.js";
import { Spin } from "antd";

import { OrderHistoryLevel, MissedOrder } from "./ProductGraph.api.types";
import {
  ProductGraphHandler,
  IProductGraphHandler,
} from "./ProductGraph.api.handler";

interface IProductGraphProps {
  skuID: number;
  handler?: IProductGraphHandler;
}

interface IProductGraphState {
  orderHistoryLevels: OrderHistoryLevel[] | undefined;
  missedOrders: MissedOrder[] | undefined;
}

export default class ProductGraph extends React.Component<
  IProductGraphProps,
  IProductGraphState
> {
  static defaultProps = {
    handler: new ProductGraphHandler(),
  };
  state: IProductGraphState = {
    orderHistoryLevels: undefined,
    missedOrders: undefined,
  };
  componentDidMount = async () => {
    const { skuID, handler } = this.props;
    const orderHistoryLevels: OrderHistoryLevel[] = await handler!.getProductOrderHistoryBySKU(
      skuID
    );
    const missedOrders: MissedOrder[] = await handler!.getProductMissedItemHistoryBySKU(
      skuID
    );

    this.setState({
      orderHistoryLevels: orderHistoryLevels.sort(
        (a: OrderHistoryLevel, b: OrderHistoryLevel) =>
          parseInt(a.createdAt.format("x")) - parseInt(b.createdAt.format("x"))
      ),
      missedOrders: missedOrders,
    });
  };

  public render() {
    const { orderHistoryLevels, missedOrders } = this.state;
    if (orderHistoryLevels === undefined || missedOrders === undefined) {
      return <Spin />;
    }
    if (orderHistoryLevels.length === 0) {
      return <p>No order history available</p>;
    }

    return (
      <div style={{ overflow: "auto" }}>
        <Plot
          data={[
            {
              x: orderHistoryLevels.map(
                (orderHistoryLevel: OrderHistoryLevel) =>
                  orderHistoryLevel.createdAt.format()
              ),
              y: orderHistoryLevels.map(
                (orderHistoryLevel: any, index: number) =>
                  orderHistoryLevel.count
              ),
              type: "scatter",
              mode: "lines+markers",
              name: "Product Stock",
              marker: { color: "blue" },
            },
            {
              x: missedOrders.map((missedOrder: MissedOrder) =>
                missedOrder.orderDate.format()
              ),
              y: missedOrders.map(
                (missedOrder: MissedOrder) => missedOrder.missedCount
              ),
              type: "scatter",
              mode: "markers",
              name: "Missed Orders",
              marker: { color: "red" },
            },
          ]}
          layout={{ title: "Warehouse Stock Level" }}
        />
      </div>
    );
  }
}
