import * as React from "react";
import { Table, Spin } from "antd";
import { DistributorProduct } from "../DistributorTable/Distributor.api.types";
import { ColumnProps } from "antd/lib/table";
import ProductGraph from "../ProductGraph/ProductGraph";
import CurrencyInput from "../util/CurrencyInput";

const columns: ColumnProps<DistributorProduct>[] = [
  {
    title: "SKU",
    dataIndex: "skuID",
    sorter: (a: any, b: any) => a.skuID - b.skuID,
    sortDirections: ["descend", "ascend"],
  },
  {
    title: "Distributor Base SKU",
    dataIndex: "distBaseSkuID",
    sorter: (a: any, b: any) => a.distBaseSkuID - b.distBaseSkuID,
    sortDirections: ["descend", "ascend"],
  },
  {
    title: "Name",
    dataIndex: "productName",
    sorter: (a: any, b: any) =>
      ("" + a.productName).localeCompare(b.productName),
    sortDirections: ["descend", "ascend"],
  },
  {
    title: "Brand",
    dataIndex: "brandName",
    sorter: (a: any, b: any) => ("" + a.brandName).localeCompare(b.brandName),
    sortDirections: ["descend", "ascend"],
  },
  {
    title: "Size Name",
    dataIndex: "sizeName",
    sorter: (a: any, b: any) => a.sizeName - b.sizeName,
    sortDirections: ["descend", "ascend"],
  },
  {
    title: "Cost Per Unit",
    dataIndex: "cost",
    sorter: (a: any, b: any) => a.cost - b.cost, //Todo: this is more complex
    sortDirections: ["descend", "ascend"],
    render: (value) => CurrencyInput({ value: value, disabled: true }),
  },
  {
    title: "OH Inventory Target Level",
    dataIndex: "onHandInventoryTargetLevel",
    sorter: (a: any, b: any) =>
      a.onHandInventoryTargetLevel - b.onHandInventoryTargetLevel,
    sortDirections: ["descend", "ascend"],
  },
  {
    title: "Quantity to Order",
    dataIndex: "qtyToOrder",
    sorter: (a: DistributorProduct, b: DistributorProduct) =>
      a.qtyToOrder - b.qtyToOrder,
    sortDirections: ["descend", "ascend"],
  },
  {
    title: "Purchase Cost",
    dataIndex: "purchaseCost",
    render: (text: string, record: DistributorProduct, index: number) =>
      CurrencyInput({ value: record.purchaseCost, disabled: true }),
  },
];

export interface IPurchaseOrderTableProps {
  products?: DistributorProduct[] | null;
}

export default class PurchaseOrderTable extends React.Component<
  IPurchaseOrderTableProps
> {
  onChange = (pagination: any, filters: any, sorter: any, extra: any) => {};
  onExpand = (expanded: boolean, record: DistributorProduct) => {};

  public render() {
    if (!this.props.products) {
      return <Spin />;
    }
    return (
      <div>
        <Table
          columns={columns}
          dataSource={this.props.products}
          onChange={this.onChange}
          rowKey={(record: DistributorProduct) =>
            record.skuID.toString() + record.key
          }
          onExpand={this.onExpand}
          expandedRowRender={(record: DistributorProduct) => (
            <ProductGraph skuID={record.skuID} />
          )}
          style={{ overflow: "auto" }}
        />
      </div>
    );
  }
}
