import csv
import matplotlib.pyplot as plt
import numpy as np
import pandas

import math
from keras.models import Sequential
from keras.layers import Dense
from keras.layers import LSTM
from sklearn.preprocessing import MinMaxScaler
from sklearn.metrics import mean_squared_error

from keras.models import load_model

import json

# https://machinelearningmastery.com/time-series-prediction-lstm-recurrent-neural-networks-python-keras/
dataframe = pandas.read_csv('data-1580417297897.csv', usecols=[1], engine='python')
dataset = dataframe.values
dataset = dataset.astype('float32')
scaler = MinMaxScaler(feature_range=(0, 1))
dataset = scaler.fit_transform(dataset)

# for i in range(len(dataset)-1, 0, -1):
# 	dataset[i][0] = max(0, dataset[i-1][0]-dataset[i][0])
# dataset = dataset[1:]
# print(dataset)

train_size = int(len(dataset) * 0.67)
test_size = len(dataset) - train_size
train, test = dataset[0:train_size,:], dataset[train_size:len(dataset),:]

# print(train)

# print(len(train), len(test))

def create_dataset(dataset, look_back=1):
	dataX, dataY = [], []
	for i in range(len(dataset)-look_back-1):
		a = dataset[i:(i+look_back), 0]
		dataX.append(a)
		dataY.append(dataset[i + look_back, 0])
	return np.array(dataX), np.array(dataY)

look_back = 10
trainX, trainY = create_dataset(train, look_back)
testX, testY = create_dataset(test, look_back)

def custom_loss(y_true, y_pred):
	error = y_true - y_pred
	# if error > 0:
	# 	error *= 2
	error = error**2
	print(error)
	return error

# reshape input to be [samples, time steps, features]
trainX = np.reshape(trainX, (trainX.shape[0], 1, trainX.shape[1]))
testX = np.reshape(testX, (testX.shape[0], 1, testX.shape[1]))
# print(trainX, trainY)

# model = load_model('iteration_two.h5')
model = Sequential()
model.add(LSTM(10, input_shape=(1, look_back)))
model.add(Dense(1))
model.compile(optimizer='adam', loss="MSE")
model.fit(trainX, trainY, epochs=100, batch_size=1, verbose=2)

# make predictions
trainPredict = model.predict(trainX)
testPredict = model.predict(testX)
# invert predictions
trainPredict = scaler.inverse_transform(trainPredict)
trainY = scaler.inverse_transform([trainY])
testPredict = scaler.inverse_transform(testPredict)
testY = scaler.inverse_transform([testY])
# calculate root mean squared error
trainScore = math.sqrt(mean_squared_error(trainY[0], trainPredict[:,0]))
print('Train Score: %.2f RMSE' % (trainScore))
testScore = math.sqrt(mean_squared_error(testY[0], testPredict[:,0]))
print('Test Score: %.2f RMSE' % (testScore))


# shift train predictions for plotting
trainPredictPlot = np.empty_like(dataset)
trainPredictPlot[:, :] = np.nan
trainPredictPlot[look_back:len(trainPredict)+look_back, :] = trainPredict
# shift test predictions for plotting
testPredictPlot = np.empty_like(dataset)
testPredictPlot[:, :] = np.nan
testPredictPlot[len(trainPredict)+(look_back*2)+1:len(dataset)-1, :] = testPredict
# plot baseline and predictions
plt.plot(scaler.inverse_transform(dataset))
plt.plot(trainPredictPlot)
plt.plot(testPredictPlot)
plt.show()


def to_javascript(all_data, test, train):
	all_values_for_js = {"dataset": [], "test": [], "train": []}

	for value in all_data.tolist():
		all_values_for_js["dataset"] += [{"y": value[0]}]
	for value in test.tolist():
		all_values_for_js["test"] += [{"y": value[0]}]
	for value in train.tolist():
		all_values_for_js["train"] += [{"y": value[0]}]

	return json.dumps(all_values_for_js)

print(to_javascript(scaler.inverse_transform(dataset), testPredictPlot, trainPredictPlot))